<?php
/*
 * system_be_edit.php
 *
 * part of pfSense plus (https://www.netgate.com)
 * Copyright (c) 2022 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 */

##|+PRIV
##|*IDENT=page-system-bootenvironments
##|*NAME=System: Boot Environments
##|*DESCR=Allow access to the 'System: Boot Environments' page.
##|*MATCH=system_be*
##|-PRIV

// pfSense includes
require_once('functions.inc');
require_once('guiconfig.inc');

// BE includes
require_once('system_be.inc');

/* keep users out if the system doesn't support BEs */
if (!be_bootenv_check()) {
	header('Location: /system_be.php');
}

$pconfig = [];
$is_new = true;

$be_name = $_REQUEST['be'] ?? null;

/* are we editing an existing boot environment or creating a new one? */
if ($pconfig = be_bootenv_get_config($be_name)) {
	$is_new = false;
} else {
	/* fallback to get the current BE */
	$pconfig['from'] = $_REQUEST['from'] ?? be_bootenv_get_current();

	/* provide a suggested name when cloning */
	$pconfig['name'] = isset($_REQUEST['from']) ? "{$pconfig['from']}_clone" : null;
}

/* handle form post */
if ($_POST && ($_POST['act'] === 'save')) {
	$res = be_bootenv_do_save($_POST);

	$input_errors	= $res['input_errors'];
	$pconfig	= $res['pconfig'];

	/* looks good... */
	if (empty($input_errors)) {
		header('Location: /system_be.php');
	}
}

$pgtitle = array(gettext('System'), gettext('Boot Environments'), gettext('Edit'));
$pglinks = array('', 'system_be.php', '@self');

include("head.inc");

if (!empty($input_errors)) {
	print_input_errors($input_errors);
}

$form = new Form(false);

/* we need a copy of the old name for renaming */
if (!$is_new) {
	$form->addGlobal(new Form_Input(
		'old_name',
		'',
		'hidden',
		$pconfig['old_name']
	));
}

$section = new Form_Section('Boot Environment');

$section->addInput(new Form_Input(
	'name',
	'*Name',
	'text',
	$pconfig['name'],
	['placeholder' => gettext('Name')]
))->setHelp(gettext('The name of the boot environment may only consist of the characters "a-z, A-Z, 0-9 and _".'));

/* only possible 'clone from' when creating a new boot environment */
if ($is_new) {
	$section->addInput(new Form_Select(
		'from',
		'*Clone From',
		$pconfig['from'],
		be_gen_select_list()
	))->setHelp(gettext('The existing boot environment to use as the source.'));
}

$section->addInput(new Form_Input(
	'descr',
	'Description',
	'text',
	$pconfig['descr'],
	['placeholder' => gettext('Description')]
))->setHelp(gettext('Boot environment description for administrative reference (not parsed).'));

$form->add($section);

$form->addGlobal(new Form_Input(
	'act',
	'',
	'hidden',
	'save'
));

print($form);
?>

<nav class="action-buttons">
	<button type="submit" id="saveform" name="saveform" class="btn btn-primary btn-sm" value="save" title="<?=gettext('Save Boot Environment')?>">
		<i class="fa fa-save icon-embed-btn"></i>
		<?=gettext('Save')?>
	</button>
</nav>

<script type="text/javascript">
//<![CDATA[
events.push(function() {
	// Save the form
	$('#saveform').click(function(event) {
		$(form).submit();
	});
});
//]]>
</script>

<?php
include('foot.inc');
